/*
 * spi_master.cpp
 *
 *  Created on: 06.03.2012
 *      Author: Felix Adam
 */

#include "spi_master.h"


#define SPI_MASTER_MOSI_PIN            ((uint8_t)(1<<5))
#define SPI_MASTER_SCK_PIN             ((uint8_t)(1<<7))



SpiMaster::SpiMaster(SPI_t* _spi, SPI_INTLVL_t _int_level)
{
  // Initialize variables
  spi_active = false;
  tx_pointer = 0;
  rx_pointer = 0;
  rx_begin = 0;
  ptr_tx_buffer = NULL;
  ptr_rx_buffer = NULL;


  // Rememper peripheral address
  spi = _spi;

  // Set interrupt level
  spi->INTCTRL = _int_level;

  // Enable peripherals
  spi->CTRL = SPI_ENABLE_bm | SPI_MASTER_bm;

  if (spi == &SPIC) {
    PORTC.DIRSET = (SPI_MASTER_MOSI_PIN | SPI_MASTER_SCK_PIN);
  }
#ifdef SPID
  else if (spi == &SPID) {
    PORTD.DIRSET = (SPI_MASTER_MOSI_PIN | SPI_MASTER_SCK_PIN);
  }
#endif
#ifdef SPIE
  else if (spi == &SPIE) {
    PORTE.DIRSET = (SPI_MASTER_MOSI_PIN | SPI_MASTER_SCK_PIN);
  }
#endif
#ifdef SPIF
  else if (spi == &SPIF) {
    PORTF.DIRSET = (SPI_MASTER_MOSI_PIN | SPI_MASTER_SCK_PIN);
  }
#endif
}



void SpiMaster::config(SPI_MODE_t _spi_mode, SPI_PRESCALER_t _spi_prescaler, uint8_t _dummy_byte)
{
  uint8_t   temp;
  
  dummy_byte = _dummy_byte;

  // Configure SPI for device to be communicated with
  temp = spi->CTRL & 0xF0;
  temp |= (_spi_mode | _spi_prescaler);
  spi->CTRL = temp;
}



void SpiMaster::setBuffer(uint8_t* _ptr_tx_buffer, uint8_t* _ptr_rx_buffer, volatile bool* _active_ptr)
{
	if (spi_active == false) {
		active_ptr = _active_ptr;
		// Save pointers and buffer sizes
		ptr_tx_buffer = _ptr_tx_buffer;
		ptr_rx_buffer = _ptr_rx_buffer;

		// Set pointers on buffer elements to 0
		tx_pointer = 0;
		rx_pointer = 0;
	}
}



void SpiMaster::readMisoLevel(PORT_t* _cs_port, uint8_t _cs_pin, bool* _level)
{
    if (spi_active == false) {
        // Setup nCS port and pin
        cs_port = _cs_port;
        cs_pin = _cs_pin;

        // Pull nCS low
        cs_port->OUTCLR = (1<<cs_pin);
        
        if (spi == &SPIC) {
           if ( (PORTC.IN & SPI_MASTER_MOSI_PIN) == 0 ) {
               *_level = false;
           }
        }
        #ifdef SPID
        else if (spi == &SPID) {
           if ( (PORTD.IN & SPI_MASTER_MOSI_PIN) == 0 ) {
               *_level = false;
           }
        }
        #endif
        #ifdef SPIE
        else if (spi == &SPIE) {
           if ( (PORTE.IN & SPI_MASTER_MOSI_PIN) == 0 ) {
               *_level = false;
           }
        }
        #endif
        #ifdef SPIF
        else if (spi == &SPIF) {
           if ( (PORTF.IN & SPI_MASTER_MOSI_PIN) == 0 ) {
               *_level = false;
           }
        }
        #endif
        
        // Pull nCS low
        cs_port->OUTSET = (1<<cs_pin);
    }
}



void SpiMaster::startTransfer(uint8_t _tx_size, uint8_t _rx_size, uint8_t _rx_begin, PORT_t* _cs_port, uint8_t _cs_pin)
{
  if (spi_active == false) {
    spi_active = true;
	*active_ptr = spi_active;

    // Setup nCS port and pin
    cs_port = _cs_port;
    cs_pin = _cs_pin;

    // Pull nCS low
    cs_port->OUTCLR = (1<<cs_pin);

    // Set pointers on buffer elements to 0
    tx_pointer = 0;
    rx_pointer = 0;

    // Assign buffer sizes and byte to start of receiving
    tx_size = _tx_size;
    rx_size = _rx_size;
    rx_begin = _rx_begin;

    spi->DATA = ptr_tx_buffer[tx_pointer++];
  }  
}



bool SpiMaster::active(void)
{
  return (spi_active);
}



void SpiMaster::handler(void)
{
	uint8_t  temp = spi->DATA;

	// Begin receiving at this byte?
	if (rx_begin < tx_pointer) {
		// Bytes to receive?
		if (rx_size != 0) {
			// Save in array
			ptr_rx_buffer[rx_pointer++] = temp;
		}
	

	if ( (tx_pointer >= tx_size) && (rx_pointer == rx_size) ) {
		spi_active = false;
		
		// Tell that reading is finished
		*active_ptr = false;

		// Pull nCS low
		cs_port->OUTSET = (1<<cs_pin);
	}

	// Send from buffer when not finished
	if (tx_pointer < tx_size) {
		spi->DATA = ptr_tx_buffer[tx_pointer++];
	}
	// Else send dummy bytes
	else if (rx_pointer != rx_size) {
		spi->DATA = dummy_byte;
		tx_pointer++;
	}
}

